package org.globus.transfer.reliable.service.cache;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.rft.generated.RFTOptionsType;
import org.globus.transfer.reliable.service.exception.RftException;

public class ConnectionParameter
{
    public static int SINGLE_TYPE = 0;
    public static int DOUBLE_TYPE = 1;
    
    private static Log logger =
        LogFactory.getLog(ConnectionParameter.class.getName());
    
    private String sourceHost;
    private String sourcePath;
    private String destinationHost;
    private String destinationPath;
    private String userDN;
    private RFTOptionsType rftOptions;
    private int sourcePort;
    private int destinationPort;
    private int type;
    private String connectionId;
    
    public ConnectionParameter(
        String         sourceHost,
        int            sourcePort,
        String         sourcePath,
        String         userDN,
        RFTOptionsType rftOptions)
    throws RftException
    {
        if (rftOptions == null) {
            throw new RftException("rftOptions must not be null");
        }
        this.type = SINGLE_TYPE;
        this.sourceHost = sourceHost;
        this.sourcePort = sourcePort;
        if (sourcePath.startsWith("/")) {
            this.sourcePath = sourcePath;
        } else {
            this.sourcePath = "/" + sourcePath;
        }
        this.userDN = userDN;
        this.rftOptions = rftOptions;
        this.destinationHost = null;
        this.destinationPort = 0;
        this.destinationPath = null;
        // generate hashCode
        this.createConnectionId();
    }
    
    public ConnectionParameter(
        String         sourceHost,
        int            sourcePort,
        String         sourcePath,
        String         destinationHost,
        int            destinationPort,
        String         destinationPath,
        String         userDN,
        RFTOptionsType rftOptions)
    throws RftException
    {
        this.type = DOUBLE_TYPE;
        this.sourceHost = sourceHost;
        this.sourcePort = sourcePort;
        if (sourcePath.startsWith("/")) {
            this.sourcePath = sourcePath;
        } else {
            this.sourcePath = "/" + sourcePath;
        }
        this.destinationHost = destinationHost;
        this.destinationPort = destinationPort;
        if (destinationPath.startsWith("/")) {
            this.destinationPath = destinationPath;
        } else {
            this.destinationPath = "/" + destinationPath;
        }
        this.userDN = userDN;
        this.rftOptions = rftOptions;
        // generate hashCode
        this.createConnectionId();
    }
    
    public int getType()
    {
        return this.type;
    }
    
    public String getDestinationHost()
    {
        return this.destinationHost;
    }

    public String getDestinationPath()
    {
        return this.destinationPath;
    }

    public int getDestinationPort()
    {
        return this.destinationPort;
    }
    
    public String getSourceHost()
    {
        return this.sourceHost;
    }

    public String getSourcePath()
    {
        return this.sourcePath;
    }

    public int getSourcePort()
    {
        return this.sourcePort;
    }

    public String getUserDN()
    {
        return this.userDN;
    }

    public RFTOptionsType getRFTOptions()
    {
        return this.rftOptions;
    }

    public String getConnectionId()
    {
        return this.connectionId;
    }

    /**
     * Create a hash value of all parameters except for the paths which
     * are not needed in the check for reuse.
     */
    private void createConnectionId()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append(userDN);
        buffer.append(sourceHost);
        buffer.append(sourcePort);
        buffer.append(rftOptions.hashCode());
        
        if (this.type == DOUBLE_TYPE) {
            buffer.append(destinationHost);
            buffer.append(destinationPort);
        }
        
        this.connectionId = buffer.toString();
        if (logger.isDebugEnabled()) {
            logger.debug("id of connection: " + connectionId);
        }
    }
}
