/*
 * Portions of this file Copyright 1999-2005 University of Chicago
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without
 * modifications, you must include this notice in the file.
 */
package org.globus.transfer.reliable.service.database;

import java.util.Calendar;

import javax.naming.InitialContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.globus.rft.generated.TransferType;
import org.globus.transfer.reliable.service.TransferWork;
import org.globus.transfer.reliable.service.ReliableFileTransferImpl;
import org.globus.transfer.reliable.service.RFTResourceManager;
import org.globus.util.I18n;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.transfer.reliable.service.database.ReliableFileTransferDbAdapter;


/**
 * Database poller for a transfer
 * TODO: Make this a singleton
 */
public class RFTDatabasePoller implements Runnable {
   
    private static Log logger = 
        LogFactory.getLog(RFTDatabasePoller.class.getName());
    
    private static I18n i18n =
        I18n.getI18n("org.globus.transfer.reliable.errors", 
                RFTDatabasePoller.class.getClassLoader());
    private boolean started = false;
    
    private long pollTime = 0;
    
    private ResourcePropertySet rpSet = null;
    
    private int requestId = 0;

    private RFTResourceManager resourceManager;

    private ReliableFileTransferDbAdapter dbAdapter;
    
    
     /* @param pollTime
     * @param rpSet
     * @param requestId
     */
    public  
    RFTDatabasePoller(
        ReliableFileTransferDbAdapter   dbAdapter,
        long                            pollTime,
        ResourcePropertySet             rpSet, 
        int                             requestId,
        RFTResourceManager              resourceManager)
    {
        this.dbAdapter = dbAdapter;
        this.pollTime = pollTime;
        this.rpSet = rpSet;
        this.requestId = requestId;
        this.resourceManager = resourceManager;
    }
    
    /* (non-Javadoc)
     * @see commonj.work.Work#release()
     */
    public void release() {
        // just for the interface
    }
    
    public boolean isDaemon() {
        return false;
    }
    
    /**
     * @return
     */
    public  boolean isStarted() {
        return this.started;
    }
    /**
     * @param pollTime
     */
    public void setPollTime(long pollTime) {
        this.pollTime = pollTime;
    }
    /*
     * main processing method
     */
    public void run() 
    {
        try
        {
            logger.debug("RFTDatabasePoller run().");
            this.started = true;

            long slp =  pollTime - Calendar.getInstance().getTimeInMillis();
            if(slp < 0) slp = 100;
            synchronized(this)
            {
                dbAdapter.addPoller(this);
                try
                {
                    logger.debug("RFTDatabasePoller waiting "+slp+" secs");
                    this.wait(slp);
                }
                catch(Exception wEx)
                {
                }
                dbAdapter.removePoller(this);
            }

            TransferType newTransfer = null;
            while(newTransfer == null && !this.resourceManager.isDone())
            {
                newTransfer = 
                    this.resourceManager.getNext();
                if (newTransfer != null) {
                    logger.debug("RFTDatabasePoller starting new transfer.");

                    //InitialContext initialCtx = new InitialContext();
                    TransferWork transferWork = new TransferWork(this.requestId,
                            this.rpSet, newTransfer);
                    transferWork.setResourceManager(this.resourceManager);
                    ReliableFileTransferImpl.getExecutor().execute(
                        transferWork);
                }
                else
                {
                    logger.error("why no transfer?");
                    synchronized(this)
                    {
                        dbAdapter.addPoller(this);
                        try
                        {
                            logger.debug("RFTDatabasePoller waiting II ");
                            this.wait(1000);
                        }
                        catch(Exception wEx)
                        {
                        }
                        dbAdapter.removePoller(this);
                    }
                }
            }
        } 
        catch(Exception ie)
        {
            logger.error(i18n.getMessage("dbPollingErr"),ie);
        } 
        this.started = false;
    }
}
